@charset "UTF-8";

.w3-theme-l5 {
background-color : #fefeff !important;
color : #000000 !important;}

.w3-theme-l4 {
background-color : #fcfbfe !important;
color : #000000 !important;}

.w3-theme-l3 {
background-color : #f6f2fd !important;
color : #290e58 !important;}

.w3-theme-l2 {
background-color : #e5d9fa !important;
color : #280c5a !important;}

.w3-theme-l1 {
background-color : #b08cf0 !important;
color : #000000 !important;}

.w3-theme-d1 {
background-color : #1f0946 !important;
color : #c1a5f3 !important;}

.w3-theme-d2 {
background-color : #15062f !important;
color : #c1a5f3 !important;}

.w3-theme-d3 {
background-color : #0e041f !important;
color : #c2a5f3 !important;}

.w3-theme-d4 {
background-color : #090315 !important;
color : #bfa6f2 !important;}

.w3-theme-d5 {
background-color : #06020e !important;
color : #bfa6f2 !important;}

.w3-theme-light {
background-color : #8952e8 !important;
color : #000000 !important;}

.w3-theme-dark {
background-color : #180734 !important;
color : #c2a5f3 !important;}

.w3-theme-action {
background-color : #0c041a !important;
color : #c2a7f1 !important;}

.w3-theme {
background-color : #2f0e69 !important;
color : #ffffff !important;}

.w3-text-theme {
color : #1ec1e1 !important;}

.w3-border-theme {
border_color : #1ec1e1 !important;}

.w3-hover-theme:hover {
background-color : #e11e73 !important;
color : #ffffff !important;}

.w3-hover-text-theme:hover {
color : #e1b31e !important;}

.w3-hover-border-theme:hover {
border-color : #e1b31e !important;}

