@charset "UTF-8";

.w3-theme-l5 {
background-color : #ffffff !important;
color : #000000 !important;}

.w3-theme-l4 {
background-color : #fffefe !important;
color : #000000 !important;}

.w3-theme-l3 {
background-color : #fefdfb !important;
color : #000000 !important;}

.w3-theme-l2 {
background-color : #fcfaf4 !important;
color : #504216 !important;}

.w3-theme-l1 {
background-color : #f6f0df !important;
color : #504116 !important;}

.w3-theme-d1 {
background-color : #c9a239 !important;
color : #000000 !important;}

.w3-theme-d2 {
background-color : #876d25 !important;
color : #ffffff !important;}

.w3-theme-d3 {
background-color : #5a4919 !important;
color : #ffffff !important;}

.w3-theme-d4 {
background-color : #3c3111 !important;
color : #e8dab0 !important;}

.w3-theme-d5 {
background-color : #28210b !important;
color : #e9dbaf !important;}

.w3-theme-light {
background-color : #f2e8cf !important;
color : #504016 !important;}

.w3-theme-dark {
background-color : #997a29 !important;
color : #ffffff !important;}

.w3-theme-action {
background-color : #4d3d14 !important;
color : #ead9ae !important;}

.w3-theme {
background-color : #e5d29f !important;
color : #000000 !important;}

.w3-text-theme {
color : #c9367b !important;}

.w3-border-theme {
border_color : #c9367b !important;}

.w3-hover-theme:hover {
background-color : #41c936 !important;
color : #ffffff !important;}

.w3-hover-text-theme:hover {
color : #36b7c9 !important;}

.w3-hover-border-theme:hover {
border-color : #36b7c9 !important;}

